@@ -1,7 +1,8 @@
+// -*- C++ -*-
 /***************************************************************************
  * blitz/array-impl.h    Definition of the Array<P_numtype, N_rank> class
  *
- * $Id: array-impl.h,v 1.14 2003/02/20 21:01:35 julianc Exp $
+ * $Id: array-impl.h,v 1.25 2005/10/13 23:46:43 julianc Exp $
  *
  * Copyright (C) 1997-2001 Todd Veldhuizen <tveldhui@oonumerics.org>
  *
@@ -37,35 +38,17 @@
 #ifndef BZ_ARRAY_H
 #define BZ_ARRAY_H
 
-#ifndef BZ_BLITZ_H
- #include <blitz/blitz.h>
-#endif
-
-#ifndef BZ_MEMBLOCK_H
- #include <blitz/memblock.h>
-#endif
-
-#ifndef BZ_RANGE_H
- #include <blitz/range.h>
-#endif
-
-#ifndef BZ_TINYVEC_H
- #include <blitz/tinyvec.h>
-#endif
+#include <blitz/blitz.h>
+#include <blitz/memblock.h>
+#include <blitz/range.h>
+#include <blitz/tinyvec.h>
 
 #ifdef BZ_ARRAY_SPACE_FILLING_TRAVERSAL
-#ifndef BZ_TRAVERSAL_H
- #include <blitz/traversal.h>
-#endif
+#include <blitz/traversal.h>
 #endif
 
-#ifndef BZ_INDEXEXPR_H
- #include <blitz/indexexpr.h>
-#endif
-
-#ifndef BZ_PRETTYPRINT_H
- #include <blitz/prettyprint.h>
-#endif
+#include <blitz/indexexpr.h>
+#include <blitz/prettyprint.h>
 
 #include <blitz/array/slice.h>     // Subarrays and slicing
 #include <blitz/array/map.h>       // Tensor index notation
@@ -95,9 +78,11 @@ class _bz_ArrayExpr;
 template<typename T_array, typename T_index>
 class IndirectArray;
 
-struct _bz_endTag {};
-
+template <typename P_numtype,int N_rank>
+void swap(Array<P_numtype,N_rank>&,Array<P_numtype,N_rank>&);
 
+template <typename P_numtype, int N_rank>
+void find(Array<TinyVector<int,N_rank>,1>&,const Array<P_numtype,N_rank>&);
 
 /*
  * Declaration of class Array
@@ -144,7 +129,7 @@ public:
     typedef ArrayIterator<T_numtype,N_rank> iterator;
     typedef ConstArrayIterator<T_numtype,N_rank> const_iterator;
 
-    enum { _bz_rank = N_rank };
+    static const int _bz_rank = N_rank;
 
     //////////////////////////////////////////////
     // Constructors                             //
@@ -156,7 +141,7 @@ public:
      */
 
     template<typename T_expr>
-    _bz_explicit Array(_bz_ArrayExpr<T_expr> expr);
+    explicit Array(_bz_ArrayExpr<T_expr> expr);
 
     /*
      * Any missing length arguments will have their value taken from the
@@ -173,7 +158,7 @@ public:
         zeroOffset_ = 0;
     }
 
-    _bz_explicit Array(int length0, 
+    explicit Array(int length0, 
         GeneralArrayStorage<N_rank> storage = GeneralArrayStorage<N_rank>())
         : storage_(storage)
     {
@@ -341,7 +326,7 @@ public:
      * Construct an array from an existing block of memory.  Ownership
      * is not acquired (this is provided for backwards compatibility).
      */
-    Array(T_numtype* _bz_restrict dataFirst, TinyVector<int, N_rank> shape,
+    Array(T_numtype* restrict dataFirst, TinyVector<int, N_rank> shape,
         GeneralArrayStorage<N_rank> storage = GeneralArrayStorage<N_rank>())
       : MemoryBlockReference<T_numtype>(product(shape), dataFirst, 
           neverDeleteData),
@@ -359,7 +344,7 @@ public:
      * given set of strides.  Ownership is not acquired (i.e. the memory
      * block will not be freed by Blitz++).
      */
-    Array(T_numtype* _bz_restrict dataFirst, TinyVector<int, N_rank> shape,
+    Array(T_numtype* restrict dataFirst, TinyVector<int, N_rank> shape,
         TinyVector<int, N_rank> stride, 
         GeneralArrayStorage<N_rank> storage = GeneralArrayStorage<N_rank>())
       : MemoryBlockReference<T_numtype>(product(shape), dataFirst, 
@@ -377,7 +362,7 @@ public:
     /*
      * Construct an array from an existing block of memory.
      */
-    Array(T_numtype* _bz_restrict dataFirst, TinyVector<int, N_rank> shape,
+    Array(T_numtype* restrict dataFirst, TinyVector<int, N_rank> shape,
         preexistingMemoryPolicy deletionPolicy,
         GeneralArrayStorage<N_rank> storage = GeneralArrayStorage<N_rank>())
       : MemoryBlockReference<T_numtype>(product(shape), dataFirst, 
@@ -398,7 +383,7 @@ public:
      * Construct an array from an existing block of memory, with a
      * given set of strides.  
      */
-    Array(T_numtype* _bz_restrict dataFirst, TinyVector<int, N_rank> shape,
+    Array(T_numtype* restrict dataFirst, TinyVector<int, N_rank> shape,
         TinyVector<int, N_rank> stride,
         preexistingMemoryPolicy deletionPolicy,
         GeneralArrayStorage<N_rank> storage = GeneralArrayStorage<N_rank>())
@@ -887,19 +872,19 @@ public:
         return dot(storage_.base(), stride_);
     }
 
-    const T_numtype* _bz_restrict     data() const
+    const T_numtype* restrict     data() const
     { return data_ + dataOffset(); }
 
-    T_numtype* _bz_restrict           data() 
+    T_numtype* restrict           data() 
     { return data_ + dataOffset(); }
 
     // These dataZero() routines refer to the point (0,0,...,0)
     // which may not be in the array if the bases are nonzero.
     
-    const T_numtype* _bz_restrict     dataZero() const
+    const T_numtype* restrict     dataZero() const
     { return data_; }
 
-    T_numtype* _bz_restrict           dataZero()
+    T_numtype* restrict           dataZero()
     { return data_; }
 
     // These dataFirst() routines refer to the element in the
@@ -920,12 +905,12 @@ public:
         return pos;
     }
     
-    const T_numtype* _bz_restrict     dataFirst() const
+    const T_numtype* restrict     dataFirst() const
     {
         return data_ + dataFirstOffset();
     }
 
-    T_numtype* _bz_restrict           dataFirst()
+    T_numtype* restrict           dataFirst()
     {
         return data_ + dataFirstOffset();
     }
@@ -945,12 +930,12 @@ public:
 
     iterator                          end()
     {
-        return iterator(*this, _bz_endTag());
+        return iterator();
     }
 
     const_iterator                    end() const
     {
-        return const_iterator(*this, _bz_endTag());
+        return const_iterator();
     }
 
     int                               extent(int rank) const
@@ -969,35 +954,23 @@ public:
         length_ = 0;
     }
  
-    _bz_bool                          isMajorRank(int rank) const
-    { return storage_.ordering(rank) == 0; }
-
-    _bz_bool                          isMinorRank(int rank) const
-    { return storage_.ordering(rank) != 0; }
-
-    _bz_bool                          isRankStoredAscending(int rank) const
-    { return storage_.isRankStoredAscending(rank); }
-
-    _bz_bool                          isStorageContiguous() const;
-
-    int                               lbound(int rank) const
-    { return base(rank); }
-
-    TinyVector<int,N_rank>            lbound() const
-    {
-        return base();
+    bool isMajorRank(int rank) const { return storage_.ordering(rank) == 0; }
+    bool isMinorRank(int rank) const { return storage_.ordering(rank) != 0; }
+    bool isRankStoredAscending(int rank) const {
+        return storage_.isRankStoredAscending(rank);
     }
 
-    int                               length(int rank) const
-    { return length_[rank]; }
+    bool isStorageContiguous() const;
 
-    const TinyVector<int, N_rank>&    length() const
-    { return length_; }
+    int                    lbound(int rank) const { return base(rank); }
+    TinyVector<int,N_rank> lbound()         const { return base(); }
 
-    void                              makeUnique();
+    int                            length(int rank) const { return length_[rank]; }
+    const TinyVector<int, N_rank>& length()         const { return length_; }
 
-    int                               numElements() const
-    { return product(length_); }
+    void makeUnique();
+
+    int numElements() const { return product(length_); }
 
     // NEEDS_WORK -- Expose the numReferences() method
     // MemoryBlockReference<T_numtype>::numReferences;
@@ -1173,39 +1146,33 @@ public:
     // Debugging routines
     //////////////////////////////////////////////
 
-    _bz_bool isInRangeForDim(int i, int d) const
-    {
+    bool isInRangeForDim(int i, int d) const {
         return i >= base(d) && (i - base(d)) < length_[d];
     }
 
-    _bz_bool isInRange(int i0) const
-    {
+    bool isInRange(int i0) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0];
     }
 
-    _bz_bool isInRange(int i0, int i1) const
-    {
+    bool isInRange(int i0, int i1) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2) const
-    {
+    bool isInRange(int i0, int i1, int i2) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
             && i3 >= base(3) && (i3 - base(3)) < length_[3];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1213,9 +1180,7 @@ public:
             && i4 >= base(4) && (i4 - base(4)) < length_[4];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4, int i5) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1224,9 +1189,7 @@ public:
             && i5 >= base(5) && (i5 - base(5)) < length_[5];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5, int i6) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4, int i5, int i6) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1236,9 +1199,8 @@ public:
             && i6 >= base(6) && (i6 - base(6)) < length_[6];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5, int i6, int i7) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4,
+        int i5, int i6, int i7) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1249,9 +1211,8 @@ public:
             && i7 >= base(7) && (i7 - base(7)) < length_[7];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5, int i6, int i7, int i8) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4,
+        int i5, int i6, int i7, int i8) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1263,9 +1224,8 @@ public:
             && i8 >= base(8) && (i8 - base(8)) < length_[8];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5, int i6, int i7, int i8, int i9) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4,
+        int i5, int i6, int i7, int i8, int i9) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1278,9 +1238,8 @@ public:
             && i9 >= base(9) && (i9 - base(9)) < length_[9];
     }
 
-    _bz_bool isInRange(int i0, int i1, int i2, int i3, int i4,
-        int i5, int i6, int i7, int i8, int i9, int i10) const
-    {
+    bool isInRange(int i0, int i1, int i2, int i3, int i4,
+        int i5, int i6, int i7, int i8, int i9, int i10) const {
         return i0 >= base(0) && (i0 - base(0)) < length_[0]
             && i1 >= base(1) && (i1 - base(1)) < length_[1]
             && i2 >= base(2) && (i2 - base(2)) < length_[2]
@@ -1294,62 +1253,57 @@ public:
             && i10 >= base(10) && (i10 - base(10)) < length_[10];
     }
 
-    _bz_bool isInRange(const T_index& index) const
-    {
+    bool isInRange(const T_index& index) const {
         for (int i=0; i < N_rank; ++i)
             if (index[i] < base(i) || (index[i] - base(i)) >= length_[i])
-                return _bz_false;
+                return false;
 
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(const T_index& BZ_DEBUG_PARAM(index)) const
-    {
+    bool assertInRange(const T_index& BZ_DEBUG_PARAM(index)) const {
         BZPRECHECK(isInRange(index), "Array index out of range: " << index
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0)) const
-    {
+    bool assertInRange(int BZ_DEBUG_PARAM(i0)) const {
         BZPRECHECK(isInRange(i0), "Array index out of range: " << i0
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0),
-        int BZ_DEBUG_PARAM(i1)) const
-    {
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1)) const {
         BZPRECHECK(isInRange(i0,i1), "Array index out of range: (" 
             << i0 << ", " << i1 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2)) const
     {
         BZPRECHECK(isInRange(i0,i1,i2), "Array index out of range: ("
             << i0 << ", " << i1 << ", " << i2 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3)) const
     {
         BZPRECHECK(isInRange(i0,i1,i2,i3), "Array index out of range: ("
             << i0 << ", " << i1 << ", " << i2 << ", " << i3 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3),
         int BZ_DEBUG_PARAM(i4)) const
     {
@@ -1358,10 +1312,10 @@ public:
             << ", " << i4 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5)) const
     {
@@ -1370,10 +1324,10 @@ public:
             << ", " << i4 << ", " << i5 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5), int BZ_DEBUG_PARAM(i6)) const
     {
@@ -1383,10 +1337,10 @@ public:
             << ", " << i4 << ", " << i5 << ", " << i6 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5), int BZ_DEBUG_PARAM(i6),
         int BZ_DEBUG_PARAM(i7)) const
@@ -1397,10 +1351,10 @@ public:
             << ", " << i4 << ", " << i5 << ", " << i6 << ", " << i7 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5), int BZ_DEBUG_PARAM(i6), int BZ_DEBUG_PARAM(i7),
         int BZ_DEBUG_PARAM(i8)) const
@@ -1412,10 +1366,10 @@ public:
             << ", " << i8 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5), int BZ_DEBUG_PARAM(i6), int BZ_DEBUG_PARAM(i7),
         int BZ_DEBUG_PARAM(i8), int BZ_DEBUG_PARAM(i9)) const
@@ -1427,10 +1381,10 @@ public:
             << ", " << i8 << ", " << i9 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
-    _bz_bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
+    bool assertInRange(int BZ_DEBUG_PARAM(i0), int BZ_DEBUG_PARAM(i1),
         int BZ_DEBUG_PARAM(i2), int BZ_DEBUG_PARAM(i3), int BZ_DEBUG_PARAM(i4),
         int BZ_DEBUG_PARAM(i5), int BZ_DEBUG_PARAM(i6), int BZ_DEBUG_PARAM(i7),
         int BZ_DEBUG_PARAM(i8), int BZ_DEBUG_PARAM(i9),
@@ -1443,7 +1397,7 @@ public:
             << ", " << i8 << ", " << i9 << ", " << i10 << ")"
             << endl << "Lower bounds: " << storage_.base() << endl
             <<         "Length:       " << length_ << endl);
-        return _bz_true;
+        return true;
     }
 
     //////////////////////////////////////////////
@@ -1451,20 +1405,20 @@ public:
     //////////////////////////////////////////////
 
     template<int N_rank2>
-    T_numtype operator()(const TinyVector<int,N_rank2>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,N_rank2>& index) const
     {
         assertInRange(index);
         return data_[dot(index, stride_)];
     }
 
     template<int N_rank2>
-    T_numtype& _bz_restrict operator()(const TinyVector<int,N_rank2>& index) 
+    T_numtype& restrict operator()(const TinyVector<int,N_rank2>& index) 
     {
         assertInRange(index);
         return data_[dot(index, stride_)];
     }
 
-    T_numtype operator()(TinyVector<int,1> index) const
+    const T_numtype& restrict operator()(TinyVector<int,1> index) const
     {
         assertInRange(index[0]);
         return data_[index[0] * stride_[0]];
@@ -1476,7 +1430,7 @@ public:
         return data_[index[0] * stride_[0]];
     }
 
-    T_numtype operator()(TinyVector<int,2> index) const
+    const T_numtype& restrict operator()(TinyVector<int,2> index) const
     {
         assertInRange(index[0], index[1]);
         return data_[index[0] * stride_[0] + index[1] * stride_[1]];
@@ -1488,7 +1442,7 @@ public:
         return data_[index[0] * stride_[0] + index[1] * stride_[1]];
     }
 
-    T_numtype operator()(TinyVector<int,3> index) const
+    const T_numtype& restrict operator()(TinyVector<int,3> index) const
     {
         assertInRange(index[0], index[1], index[2]);
         return data_[index[0] * stride_[0] + index[1] * stride_[1]
@@ -1502,7 +1456,7 @@ public:
             + index[2] * stride_[2]];
     }
 
-    T_numtype operator()(const TinyVector<int,4>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,4>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3]);
         return data_[index[0] * stride_[0] + index[1] * stride_[1]
@@ -1516,7 +1470,7 @@ public:
             + index[2] * stride_[2] + index[3] * stride_[3]];
     }
 
-    T_numtype operator()(const TinyVector<int,5>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,5>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4]);
@@ -1534,7 +1488,7 @@ public:
             + index[4] * stride_[4]];
     }
 
-    T_numtype operator()(const TinyVector<int,6>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,6>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5]);
@@ -1552,7 +1506,7 @@ public:
             + index[4] * stride_[4] + index[5] * stride_[5]];
     }
 
-    T_numtype operator()(const TinyVector<int,7>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,7>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5], index[6]);
@@ -1572,7 +1526,7 @@ public:
             + index[6] * stride_[6]];
     }
 
-    T_numtype operator()(const TinyVector<int,8>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,8>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5], index[6], index[7]);
@@ -1592,7 +1546,7 @@ public:
             + index[6] * stride_[6] + index[7] * stride_[7]];
     }
 
-    T_numtype operator()(const TinyVector<int,9>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,9>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5], index[6], index[7], index[8]);
@@ -1614,7 +1568,7 @@ public:
             + index[8] * stride_[8]];
     }
 
-    T_numtype operator()(const TinyVector<int,10>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,10>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5], index[6], index[7], index[8], index[9]);
@@ -1636,7 +1590,7 @@ public:
             + index[8] * stride_[8] + index[9] * stride_[9]];
     }
 
-    T_numtype operator()(const TinyVector<int,11>& index) const
+    const T_numtype& restrict operator()(const TinyVector<int,11>& index) const
     {
         assertInRange(index[0], index[1], index[2], index[3],
             index[4], index[5], index[6], index[7], index[8], index[9],
@@ -1662,59 +1616,59 @@ public:
             + index[10] * stride_[10]];
     }
 
-    T_numtype operator()(int i0) const
+    const T_numtype& restrict operator()(int i0) const
     { 
         assertInRange(i0);
         return data_[i0 * stride_[0]]; 
     }
 
-    T_numtype& _bz_restrict operator()(int i0) 
+    T_numtype& restrict operator()(int i0) 
     {
         assertInRange(i0);
         return data_[i0 * stride_[0]];
     }
 
-    T_numtype operator()(int i0, int i1) const
+    const T_numtype& restrict operator()(int i0, int i1) const
     { 
         assertInRange(i0, i1);
         return data_[i0 * stride_[0] + i1 * stride_[1]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1)
+    T_numtype& restrict operator()(int i0, int i1)
     {
         assertInRange(i0, i1);
         return data_[i0 * stride_[0] + i1 * stride_[1]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2) const
+    const T_numtype& restrict operator()(int i0, int i1, int i2) const
     {
         assertInRange(i0, i1, i2);
         return data_[i0 * stride_[0] + i1 * stride_[1]
             + i2 * stride_[2]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2) 
+    T_numtype& restrict operator()(int i0, int i1, int i2) 
     {
         assertInRange(i0, i1, i2);
         return data_[i0 * stride_[0] + i1 * stride_[1]
             + i2 * stride_[2]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3) const
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3) const
     {
         assertInRange(i0, i1, i2, i3);
         return data_[i0 * stride_[0] + i1 * stride_[1]
             + i2 * stride_[2] + i3 * stride_[3]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3)
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3)
     {
         assertInRange(i0, i1, i2, i3);
         return data_[i0 * stride_[0] + i1 * stride_[1]
             + i2 * stride_[2] + i3 * stride_[3]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4) const
     {
         assertInRange(i0, i1, i2, i3, i4);
@@ -1722,7 +1676,7 @@ public:
             + i2 * stride_[2] + i3 * stride_[3] + i4 * stride_[4]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4)
     {
         assertInRange(i0, i1, i2, i3, i4);
@@ -1730,7 +1684,7 @@ public:
             + i2 * stride_[2] + i3 * stride_[3] + i4 * stride_[4]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5);
@@ -1739,7 +1693,7 @@ public:
             + i5 * stride_[5]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5)
     {
         assertInRange(i0, i1, i2, i3, i4, i5);
@@ -1748,7 +1702,7 @@ public:
             + i5 * stride_[5]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6);
@@ -1757,7 +1711,7 @@ public:
             + i5 * stride_[5] + i6 * stride_[6]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6)
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6);
@@ -1766,7 +1720,7 @@ public:
             + i5 * stride_[5] + i6 * stride_[6]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7);
@@ -1775,7 +1729,7 @@ public:
             + i5 * stride_[5] + i6 * stride_[6] + i7 * stride_[7]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7)
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7);
@@ -1784,7 +1738,7 @@ public:
             + i5 * stride_[5] + i6 * stride_[6] + i7 * stride_[7]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8);
@@ -1794,7 +1748,7 @@ public:
             + i8 * stride_[8]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8)
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8);
@@ -1804,7 +1758,7 @@ public:
             + i8 * stride_[8]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8, int i9) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8, i9);
@@ -1814,7 +1768,7 @@ public:
             + i8 * stride_[8] + i9 * stride_[9]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8, int i9)
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8, i9);
@@ -1824,7 +1778,7 @@ public:
             + i8 * stride_[8] + i9 * stride_[9]];
     }
 
-    T_numtype operator()(int i0, int i1, int i2, int i3,
+    const T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8, int i9, int i10) const
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8, 
@@ -1835,7 +1789,7 @@ public:
             + i8 * stride_[8] + i9 * stride_[9] + i10 * stride_[10]];
     }
 
-    T_numtype& _bz_restrict operator()(int i0, int i1, int i2, int i3,
+    T_numtype& restrict operator()(int i0, int i1, int i2, int i3,
         int i4, int i5, int i6, int i7, int i8, int i9, int i10)
     {
         assertInRange(i0, i1, i2, i3, i4, i5, i6, i7, i8, 
@@ -1949,7 +1903,7 @@ public:
      * the appropriate cases exist above.
      */
 
-#ifdef BZ_PARTIAL_ORDERING
+#ifdef BZ_HAVE_PARTIAL_ORDERING
 
     template<typename T1, typename T2>
     typename SliceInfo<T_numtype,T1,T2>::T_slice
@@ -2047,7 +2001,7 @@ public:
         return slice(noConst(), r1, r2, r3, r4, r5, r6, r7, r8, r9, r10, r11);
     }
 
-#endif // BZ_PARTIAL_ORDERING
+#endif // BZ_HAVE_PARTIAL_ORDERING
 
     /*
      * These versions of operator() are provided to support tensor-style
@@ -2188,15 +2142,30 @@ public:
      * multicomponent_traits.
      */
 
-    // NEEDS_WORK: const version
     Array<typename multicomponent_traits<T_numtype>::T_element,N_rank>
-    operator[](int component)
-    {
-        typedef typename multicomponent_traits<T_numtype>::T_element 
-            T_compType;
+    operator[](const unsigned component) {
+        typedef typename multicomponent_traits<T_numtype>::T_element T_compType;
 
-        return extractComponent(T_compType(), 
-            component, multicomponent_traits<T_numtype>::numComponents);
+        return extractComponent(T_compType(),component,
+                                multicomponent_traits<T_numtype>::numComponents);
+    }
+
+    const Array<typename multicomponent_traits<T_numtype>::T_element,N_rank>
+    operator[](const unsigned component) const {
+        typedef typename multicomponent_traits<T_numtype>::T_element T_compType;
+
+        return extractComponent(T_compType(),component,
+                                multicomponent_traits<T_numtype>::numComponents);
+    }
+
+    Array<typename multicomponent_traits<T_numtype>::T_element,N_rank>
+    operator[](const int component) {
+        return operator[](static_cast<unsigned>(component));
+    }
+
+    const Array<typename multicomponent_traits<T_numtype>::T_element,N_rank>
+    operator[](const int component) const {
+        return operator[](static_cast<unsigned>(component));
     }
 
     //////////////////////////////////////////////
@@ -2349,11 +2318,9 @@ public:
         T_expr expr, T_update);
 
 
-    T_numtype* _bz_restrict getInitializationIterator()
-    { return dataFirst(); }
+    T_numtype* restrict getInitializationIterator() { return dataFirst(); }
 
-    _bz_bool canCollapse(int outerRank, int innerRank) const
-    { 
+    bool canCollapse(int outerRank, int innerRank) const { 
 #ifdef BZ_DEBUG_TRAVERSE
         BZ_DEBUG_MESSAGE("stride(" << innerRank << ")=" << stride(innerRank)
           << ", extent()=" << extent(innerRank) << ", stride(outerRank)="
@@ -2500,6 +2467,37 @@ ostream& operator<<(ostream&, const Array<T_numtype,N_rank>&);
 template<typename T_numtype, int N_rank>
 istream& operator>>(istream& is, Array<T_numtype,N_rank>& x);
 
+template <typename P_numtype,int N_rank>
+void swap(Array<P_numtype,N_rank>& a,Array<P_numtype,N_rank>& b) {
+    Array<P_numtype,N_rank> c(a);
+    a.reference(b);
+    b.reference(c);
+}
+
+template <typename P_expr>
+void find(Array<TinyVector<int,P_expr::rank>,1>& indices,
+          const _bz_ArrayExpr<P_expr>& expr) {
+    find(indices,
+         static_cast< Array<typename P_expr::T_numtype,P_expr::rank> >(expr));
+}
+
+template <typename P_numtype, int N_rank>
+void find(Array<TinyVector<int,N_rank>,1>& indices,
+          const Array<P_numtype,N_rank>& exprVals) {
+    indices.resize(exprVals.size());
+    typename Array<P_numtype,N_rank>::const_iterator it, end = exprVals.end();
+    int j=0; 
+    for (it = exprVals.begin(); it != end; ++it)
+        if (*it) 
+            indices(j++) = it.position();
+    if (j) 
+        indices.resizeAndPreserve(j);
+    else 
+        indices.free();
+    return;
+}
+
+
 BZ_NAMESPACE_END
 
 /*
@@ -2527,4 +2525,3 @@ BZ_NAMESPACE_END
 #include <blitz/array/stencils.h>   // Stencil objects
 
 #endif // BZ_ARRAY_H
-
