@@ -1,7 +1,8 @@
+// -*- C++ -*-
 /***************************************************************************
  * blitz/meta/metaprog.h   Useful metaprogram declarations
  *
- * $Id: metaprog.h,v 1.5 2003/12/11 03:44:23 julianc Exp $
+ * $Id: metaprog.h,v 1.6 2005/05/07 04:17:57 julianc Exp $
  *
  * Copyright (C) 1997-2001 Todd Veldhuizen <tveldhui@oonumerics.org>
  *
@@ -45,7 +46,7 @@ template<typename T> inline T operator*(const T& a, _bz_meta_nullOperand)
 template<int N1, int N2>
 class _bz_meta_max {
 public:
-    enum { max = (N1 > N2) ? N1 : N2 };
+    static const int max = (N1 > N2) ? N1 : N2;
 };
 
 // MetaMin
@@ -53,7 +54,7 @@ public:
 template<int N1, int N2>
 class _bz_meta_min {
 public:
-    enum { min = (N1 < N2) ? N1 : N2 };
+    static const int min = (N1 < N2) ? N1 : N2;
 };
 
 BZ_NAMESPACE_END 
