@@ -1,7 +1,8 @@
+// -*- C++ -*-
 /***************************************************************************
  * blitz/tinyvec.h      Declaration of the TinyVector<T, N> class
  *
- * $Id: tinyvec.h,v 1.7 2003/12/11 03:44:22 julianc Exp $
+ * $Id: tinyvec.h,v 1.9 2005/10/06 23:23:40 julianc Exp $
  *
  * Copyright (C) 1997-2001 Todd Veldhuizen <tveldhui@oonumerics.org>
  *
@@ -26,18 +27,9 @@
 #ifndef BZ_TINYVEC_H
 #define BZ_TINYVEC_H
 
-#ifndef BZ_BLITZ_H
- #include <blitz/blitz.h>
-#endif
-
-#ifndef BZ_RANGE_H
- #include <blitz/range.h>
-#endif
-
-#ifndef BZ_LISTINIT_H
- #include <blitz/listinit.h>
-#endif
-
+#include <blitz/blitz.h>
+#include <blitz/range.h>
+#include <blitz/listinit.h>
 #include <blitz/tiny.h>
 
 BZ_NAMESPACE(blitz)
@@ -76,8 +68,8 @@ class _bz_matrixVectorProduct;
 
 template<typename P_numtype, int N_length>
 class TinyVector {
-
 public:
+
     //////////////////////////////////////////////
     // Public Types
     //////////////////////////////////////////////
@@ -86,22 +78,23 @@ public:
     typedef TinyVector<T_numtype,N_length>               T_vector;
     typedef TinyVectorIter<T_numtype,N_length,1>         T_iterator;
     typedef TinyVectorIterConst<T_numtype,N_length,1>    T_constIterator;
-    typedef T_iterator iterator;
-    typedef T_constIterator const_iterator;
+    typedef T_numtype*                                   iterator;
+    typedef const T_numtype*                             const_iterator;
     enum { numElements = N_length };
 
-    TinyVector()
-    { }
-
-    ~TinyVector() 
-    { }
+    TinyVector()  { }
+    ~TinyVector() { }
 
     inline TinyVector(const TinyVector<T_numtype,N_length>& x);
 
     template <typename T_numtype2>
     inline TinyVector(const TinyVector<T_numtype2,N_length>& x);
 
-    inline TinyVector(T_numtype initValue);
+    inline TinyVector(const T_numtype initValue);
+
+    inline TinyVector(const T_numtype x[]) {
+        memcpy(data_,x,N_length*sizeof(T_numtype));
+    }
 
     TinyVector(T_numtype x0, T_numtype x1)
     {
@@ -224,14 +217,14 @@ public:
     template<typename P_expr>
     inline TinyVector(_bz_VecExpr<P_expr> expr);
 
-    T_iterator begin()
-    { return T_iterator(*this); }
+    T_iterator      beginFast()       { return T_iterator(*this);      }
+    T_constIterator beginFast() const { return T_constIterator(*this); }
 
-    T_constIterator begin() const
-    { return T_constIterator(*this); }
+    iterator       begin()       { return data_; }
+    const_iterator begin() const { return data_; }
 
-    // T_iterator end();
-    // T_constIterator end() const;
+    iterator       end()       { return data_ + N_length; }
+    const_iterator end() const { return data_ + N_length; }
 
     T_numtype * restrict data()
     { return data_; }
@@ -254,23 +247,23 @@ public:
     // disappear in future releases.
     /////////////////////////////////////////////
 
-    unsigned        _bz_suggestLength() const
+    unsigned _bz_suggestLength() const
     { return N_length; }
 
-    bool        _bz_hasFastAccess() const
+    bool _bz_hasFastAccess() const
     { return true; }
 
-    T_numtype& restrict     _bz_fastAccess(unsigned i)
+    T_numtype& restrict _bz_fastAccess(unsigned i)
     { return data_[i]; }
 
-    T_numtype       _bz_fastAccess(unsigned i) const
+    T_numtype _bz_fastAccess(unsigned i) const
     { return data_[i]; }
 
     template<typename P_expr, typename P_updater>
     void _bz_assign(P_expr, P_updater);
 
     _bz_VecExpr<T_constIterator> _bz_asVecExpr() const
-    { return _bz_VecExpr<T_constIterator>(begin()); }
+    { return _bz_VecExpr<T_constIterator>(beginFast()); }
    
     //////////////////////////////////////////////
     // Subscripting operators
@@ -284,7 +277,7 @@ public:
         return true;
     }
 
-    T_numtype operator()(unsigned i) const
+    const T_numtype& operator()(unsigned i) const
     {
         BZPRECONDITION(lengthCheck(i));
         return data_[i];
@@ -296,7 +289,7 @@ public:
         return data_[i];
     }
 
-    T_numtype operator[](unsigned i) const
+    const T_numtype& operator[](unsigned i) const
     {
         BZPRECONDITION(lengthCheck(i));
         return data_[i];
@@ -318,17 +311,17 @@ public:
         return ListInitializationSwitch<T_vector,T_numtype*>(*this, x);
     }
 
-    T_vector& initialize(T_numtype);
-    T_vector& operator+=(T_numtype);
-    T_vector& operator-=(T_numtype);
-    T_vector& operator*=(T_numtype);
-    T_vector& operator/=(T_numtype);
-    T_vector& operator%=(T_numtype);
-    T_vector& operator^=(T_numtype);
-    T_vector& operator&=(T_numtype);
-    T_vector& operator|=(T_numtype);
-    T_vector& operator>>=(int);
-    T_vector& operator<<=(int);
+    T_vector& initialize(const T_numtype);
+    T_vector& operator+=(const T_numtype);
+    T_vector& operator-=(const T_numtype);
+    T_vector& operator*=(const T_numtype);
+    T_vector& operator/=(const T_numtype);
+    T_vector& operator%=(const T_numtype);
+    T_vector& operator^=(const T_numtype);
+    T_vector& operator&=(const T_numtype);
+    T_vector& operator|=(const T_numtype);
+    T_vector& operator>>=(const int);
+    T_vector& operator<<=(const int);
 
     template<typename P_numtype2> 
     T_vector& operator=(const TinyVector<P_numtype2, N_length> &);
@@ -403,17 +396,17 @@ public:
     T_vector& operator<<=(const VectorPick<P_numtype2> &);
 
     // Range operand
-    T_vector& operator=(Range);
-    T_vector& operator+=(Range);
-    T_vector& operator-=(Range);
-    T_vector& operator*=(Range);
-    T_vector& operator/=(Range);
-    T_vector& operator%=(Range);
-    T_vector& operator^=(Range);
-    T_vector& operator&=(Range);
-    T_vector& operator|=(Range);
-    T_vector& operator>>=(Range);
-    T_vector& operator<<=(Range);
+    T_vector& operator=(const Range&);
+    T_vector& operator+=(const Range&);
+    T_vector& operator-=(const Range&);
+    T_vector& operator*=(const Range&);
+    T_vector& operator/=(const Range&);
+    T_vector& operator%=(const Range&);
+    T_vector& operator^=(const Range&);
+    T_vector& operator&=(const Range&);
+    T_vector& operator|=(const Range&);
+    T_vector& operator>>=(const Range&);
+    T_vector& operator<<=(const Range&);
 
     T_numtype* restrict getInitializationIterator()
     { return dataFirst(); }
