@@ -1,13 +1,11 @@
 /*****************************************************************************
  * outer.cpp        Blitz++ outer product (tensor notation) example
- * $Id: outer.cpp,v 1.4 2003/01/13 11:17:06 patricg Exp $
+ * $Id: outer.cpp,v 1.5 2004/10/07 01:23:26 julianc Exp $
  *****************************************************************************/
 
 #include <blitz/array.h>
 
-#ifdef BZ_NAMESPACES
-using namespace blitz;
-#endif
+BZ_USING_NAMESPACE(blitz)
 
 int main()
 {
